# Java Multithreading and Thread Model Presentation

## Introduction to Java Multithreading

### What is Multithreading?

- **Multithreading** is a process of executing multiple threads simultaneously.
- A thread is a lightweight sub-process, the smallest unit of processing.
- **Java** provides built-in support for multithreaded programming using the `Thread` class and `Runnable` interface.

### Why Multithreading?

- **Concurrency**: Improve application performance by executing independent tasks in parallel.
- **Resource Utilization**: Efficiently utilize CPU resources.
- **Responsiveness**: Increase application responsiveness (especially UI).

### Multithreading Design Principles

1. **Thread Creation**
   - Use the `Thread` class: Extend `Thread` and override `run()` method.
   - Use the `Runnable` interface: Implement `Runnable` and override `run()` method.

2. **Synchronization**
   - Ensure thread safety when accessing shared resources.
   - Use `synchronized` keyword to lock resources.
   - Java provides synchronized blocks/methods and concurrent collections.

3. **Thread Coordination**
   - Techniques to coordinate communication between threads.
   - Use `wait()`, `notify()`, and `notifyAll()` methods.
   - CountDownLatch, CyclicBarrier, Semaphore are high-level constructs.

4. **Thread Pooling**
   - Reusing existing threads to execute tasks.
   - Use `ExecutorService` to manage thread pools.
   - Benefits: better resource management, reduced latency.

5. **Atomic Operations**
   - Ensure operations on variables are performed atomically.
   - Use classes from `java.util.concurrent.atomic` package (e.g., `AtomicInteger`).

### Code Example: Creating Threads

#### Using `Thread` Class

```java
class MyThread extends Thread {
    @Override
    public void run() {
        System.out.println("Thread is running using Thread class.");
    }
}

public class ThreadExample1 {
    public static void main(String[] args) {
        MyThread thread = new MyThread();
        thread.start();
    }
}
```

#### Using `Runnable` Interface

```java
class MyRunnable implements Runnable {
    @Override
    public void run() {
        System.out.println("Thread is running using Runnable interface.");
    }
}

public class ThreadExample2 {
    public static void main(String[] args) {
        Thread thread = new Thread(new MyRunnable());
        thread.start();
    }
}
```

### References for Multithreading Design Principles

- **Books**: 
  - "Java Concurrency in Practice" by Brian Goetz.
  
- **Articles/Documentation**:
  - [Java Tutorials - Concurrency](https://docs.oracle.com/javase/tutorial/essential/concurrency/)
  - [Java Concurrency & Multithreading Basics](https://www.baeldung.com/java-concurrency)

## Java Threads Model

### Java Thread Lifecycle

- **New**: Thread is created but not yet started.
- **Runnable**: After calling `start()`, thread is ready to run.
- **Blocked/Waiting**: Thread is blocked or waiting temporarily.
- **Timed Waiting**: Thread waits for another thread to perform a task up to a specified waiting time.
- **Terminated**: Thread has finished its execution.

### Thread Model Details

1. **Priority**
   - Each thread has a priority.
   - Threads with higher priority are executed in preference to threads with lower priority.

2. **Daemon Threads**
   - Low-priority threads that run in the background performing tasks like garbage collection.
   - Use `setDaemon(true)` to mark a thread as daemon.

3. **Thread Safety**
   - Critical when multiple threads access shared resources.
   - Guard critical code sections using synchronization.

4. **Thread Interruptions**
   - Mechanism to signal a thread to stop its current activities.
   - Use `interrupt()` method and check `isInterrupted()` status.

5. **Locking Mechanisms**
   - Intrinsic locks and explicit locks (`ReentrantLock`).

### Code Example: Thread Lifecycle and Priority

```java
class LifecycleExample implements Runnable {
    @Override
    public void run() {
        System.out.println(Thread.currentThread().getName() + " is in state: " + Thread.currentThread().getState());
        try {
            Thread.sleep(200);
        } catch (InterruptedException e) {
            System.out.println(Thread.currentThread().getName() + " was interrupted.");
        }
        System.out.println(Thread.currentThread().getName() + " is in state: " + Thread.currentThread().getState());
    }
}

public class ThreadLifecycleDemo {
    public static void main(String[] args) {
        Thread thread = new Thread(new LifecycleExample(), "DemoThread");
        System.out.println(thread.getName() + " is in state: " + thread.getState());
        thread.start();
        System.out.println(thread.getName() + " is in state: " + thread.getState());
    }
}
```

### References for Java Threads Model

- **Books**: 
  - "Effective Java" by Joshua Bloch.
  
- **Articles/Documentation**:
  - [Java Official Documentation on Threads](https://docs.oracle.com/javase/10/docs/api/java/lang/Thread.html)
  - [Concurrency / Basic Multi-threading in Java](https://www.geeksforgeeks.org/multithreading-in-java/)

This presentation provides a comprehensive understanding of Java's multithreading design principles and the threading model. For further reading, explore the referenced materials, as they provide deeper insights and practical knowledge.